@@ -1,4 +1,5 @@
 import sys
+from inspect import ArgSpec
 
 if sys.version_info[0] == 3:  # pragma: no cover
     from inspect import getfullargspec  # @UnresolvedImport
@@ -7,7 +8,27 @@ else:  # pragma: no cover
     from collections import namedtuple
     FullArgSpec = namedtuple('FullArgSpec', 'args varargs varkw defaults'
                              ' kwonlyargs kwonlydefaults annotations')
-    from inspect import getargspec
+    from inspect import getargspec as _getargspec
+    
+    def getargspec(function):
+        # print 'hasattr im_func', hasattr(function, 'im_func')
+        if hasattr(function, 'im_func'):
+            # print('this is a special function : %s' % function)
+            # For methods or classmethods drop the first
+            # argument from the returned list because
+            # python supplies that automatically for us.
+            # Note that this differs from what
+            # inspect.getargspec() returns for methods.
+            # NB: We use im_func so we work with
+            #     instancemethod objects also.
+            x = _getargspec(function.im_func)
+            new_args = x.args[1:]
+            spec = ArgSpec(args=new_args, varargs=x.varargs,
+                           keywords=x.keywords, defaults=x.defaults)
+            return spec
+        
+        # print 'calling normal %s' % function
+        return _getargspec(function)
 
     def getfullargspec(function):
         spec = getargspec(function)
@@ -35,7 +56,7 @@ else:  # pragma: no cover
         names of the * and ** arguments, if any), and values the respective bound
         values from 'positional' and 'named'."""
         args, varargs, varkw, defaults, \
-            kwonlyargs, kwonlydefaults, annotations = getfullargspec(func) #@UnusedVariable
+            kwonlyargs, kwonlydefaults, annotations = getfullargspec(func)  # @UnusedVariable
 
         if kwonlyargs:
             raise ValueError("I'm sorry, I don't have the logic to use kwonlyargs. "
@@ -46,6 +67,7 @@ else:  # pragma: no cover
 
         # The following closures are basically because of tuple parameter unpacking.
         assigned_tuple_params = []
+        
         def assign(arg, value):
             if isinstance(arg, str):
                 arg2value[arg] = value
@@ -56,7 +78,7 @@ else:  # pragma: no cover
                     try:
                         subvalue = next(value)
                     except StopIteration:
-                        raise ValueError('need more than %d %s to unpack' %
+                        raise ValueError('need more than %d %s to unpack' % 
                                          (i, 'values' if i > 1 else 'value'))
                     assign(subarg, subvalue)
                 try:
@@ -65,6 +87,7 @@ else:  # pragma: no cover
                     pass
                 else:
                     raise ValueError('too many values to unpack')
+        
         def is_assigned(arg):
             if isinstance(arg, str):
                 return arg in arg2value
@@ -94,7 +117,7 @@ else:  # pragma: no cover
                 f_name, 'at most' if defaults else 'exactly', num_args,
                 'arguments' if num_args > 1 else 'argument', num_total))
         elif num_args == 0 and num_total:
-            raise TypeError('%s() takes no arguments (%d given)' %
+            raise TypeError('%s() takes no arguments (%d given)' % 
                             (f_name, num_total))
         for arg in args:
             if isinstance(arg, str) and arg in named:
@@ -103,7 +126,7 @@ else:  # pragma: no cover
                                     "argument '%s'" % (f_name, arg))
                 else:
                     assign(arg, named.pop(arg))
-        if defaults:    # fill in any missing values with the defaults
+        if defaults:  # fill in any missing values with the defaults
             for arg, value in zip(args[-num_defaults:], defaults):
                 if not is_assigned(arg):
                     assign(arg, value)
@@ -113,7 +136,7 @@ else:  # pragma: no cover
             unexpected = next(iter(named))
             if isinstance(unexpected, unicode):
                 unexpected = unexpected.encode(sys.getdefaultencoding(), 'replace')
-            raise TypeError("%s() got an unexpected keyword argument '%s'" %
+            raise TypeError("%s() got an unexpected keyword argument '%s'" % 
                             (f_name, unexpected))
         unassigned = num_args - len([arg for arg in args if is_assigned(arg)])
         if unassigned:
